@@ -1,26 +1,26 @@
-/************************** BEGIN jack-dsp.h **************************/
-/************************************************************************
+/************************** BEGIN jack-dsp.h **************************
  FAUST Architecture File
- Copyright (C) 2003-2017 GRAME, Centre National de Creation Musicale
+ Copyright (C) 2003-2022 GRAME, Centre National de Creation Musicale
  ---------------------------------------------------------------------
- This Architecture section is free software; you can redistribute it
- and/or modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; either version 3 of
- the License, or (at your option) any later version.
+ This program is free software; you can redistribute it and/or modify
+ it under the terms of the GNU Lesser General Public License as published by
+ the Free Software Foundation; either version 2.1 of the License, or
+ (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ GNU Lesser General Public License for more details.
  
- You should have received a copy of the GNU General Public License
- along with this program; If not, see <http://www.gnu.org/licenses/>.
+ You should have received a copy of the GNU Lesser General Public License
+ along with this program; if not, write to the Free Software
+ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  
  EXCEPTION : As a special exception, you may create a larger work
  that contains this FAUST architecture section and distribute
  that work under terms of your choice, so long as this FAUST
  architecture section is not modified.
- ************************************************************************/
+ **********************************************************************/
 
 #ifndef __jack_dsp__
 #define __jack_dsp__
@@ -426,7 +426,7 @@ class jackaudio : public audio {
 
 // Add JACK MIDI
 
-class jackaudio_midi : public jackaudio, public jack_midi_handler {
+class jackaudio_midi : public jackaudio, public jack_midi {
         
     protected:
     
@@ -502,7 +502,7 @@ class jackaudio_midi : public jackaudio, public jack_midi_handler {
     public:
         
         jackaudio_midi(bool auto_connect = true, bool is_polling = false)
-        :jackaudio(auto_connect), jack_midi_handler("JACKMidi"), fPolling(is_polling)
+        :jackaudio(auto_connect), jack_midi("JACKMidi"), fPolling(is_polling)
         {}
         
         virtual ~jackaudio_midi()
