@@ -1,25 +1,26 @@
-/************************************************************************
+/************************** BEGIN SimpleParser.h *********************
  FAUST Architecture File
- Copyright (C) 2003-2017 GRAME, Centre National de Creation Musicale
+ Copyright (C) 2003-2022 GRAME, Centre National de Creation Musicale
  ---------------------------------------------------------------------
- This Architecture section is free software; you can redistribute it
- and/or modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; either version 3 of
- the License, or (at your option) any later version.
+ This program is free software; you can redistribute it and/or modify
+ it under the terms of the GNU Lesser General Public License as published by
+ the Free Software Foundation; either version 2.1 of the License, or
+ (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ GNU Lesser General Public License for more details.
  
- You should have received a copy of the GNU General Public License
- along with this program; If not, see <http://www.gnu.org/licenses/>.
+ You should have received a copy of the GNU Lesser General Public License
+ along with this program; if not, write to the Free Software
+ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  
  EXCEPTION : As a special exception, you may create a larger work
  that contains this FAUST architecture section and distribute
  that work under terms of your choice, so long as this FAUST
  architecture section is not modified.
- ************************************************************************/
+ ********************************************************************/
 
 #ifndef SIMPLEPARSER_H
 #define SIMPLEPARSER_H
@@ -34,8 +35,10 @@
 #include <vector>
 #include <map>
 #include <string>
+#include <cmath>
 #include <fstream>
-#include <iostream>
+#include <sstream>
+#include <stdio.h> // We use the lighter fprintf code
 #include <ctype.h>
 
 #ifndef _WIN32
@@ -47,33 +50,17 @@ struct itemInfo {
     std::string label;
     std::string url;
     std::string address;
-    std::string index;
-    std::string init;
-    std::string min;
-    std::string max;
-    std::string step;
+    int index;
+    double init;
+    double fmin;
+    double fmax;
+    double step;
     std::vector<std::pair<std::string, std::string> > meta;
+    
+    itemInfo():index(0), init(0.), fmin(0.), fmax(0.), step(0.)
+    {}
 };
 
-/*
-// Menu {'low' : 440.0; 'mid' : 880.0; 'hi' : 1760.0}
-static bool parseMenuList(const char*& p, std::vector<std::string>& names, std::vector<double>& values);
-static bool parseMenuItem(const char*& p, std::string& name, double& value);
-
-// Menu {'foo.wav'; 'bar.wav'}
-static bool parseMenuList2(const char*& p, std::vector<std::string>& names, bool debug = false);
-static bool parseMenuItem2(const char*& p, std::string& name);
-
-static void skipBlank(const char*& p);
-static bool parseChar(const char*& p, char x);
-static bool parseWord(const char*& p, const char* w);
-static bool parseString(const char*& p, char quote, std::string& s);
-static bool parseSQString(const char*& p, std::string& s);
-static bool parseDQString(const char*& p, std::string& s);
-static bool parseDouble(const char*& p, double& x);
-static bool parseList(const char*& p, std::vector<std::string>& items);
-*/
-
 // ---------------------------------------------------------------------
 //                          Elementary parsers
 // ---------------------------------------------------------------------
@@ -81,7 +68,7 @@ static bool parseList(const char*& p, std::vector<std::string>& items);
 // Report a parsing error
 static bool parseError(const char*& p, const char* errmsg)
 {
-    std::cerr << "Parse error : " << errmsg << " here : " << p << std::endl;
+    fprintf(stderr, "Parse error : %s here : %s\n", errmsg, p);
     return true;
 }
 
@@ -143,32 +130,40 @@ static bool parseWord(const char*& p, const char* w)
 }
 
 /**
- * @brief parseDouble : parse number [s]dddd[.dddd] and store the result in x
+ * @brief parseDouble : parse number [s]dddd[.dddd] or [s]d[.dddd][E|e][s][dddd] and store the result in x
  * @param p the string to parse, then the remaining string
  * @param x the float number found if any
  * @return true if a float number was found at the begin of p
  */
 static bool parseDouble(const char*& p, double& x)
 {
-    double sign = +1.0;    // sign of the number
+    double sign = 1.0;     // sign of the number
     double ipart = 0;      // integral part of the number
     double dpart = 0;      // decimal part of the number before division
     double dcoef = 1.0;    // division factor for the decimal part
+    double expsign = 1.0;  // sign of the E|e part
+    double expcoef = 0.0;  // multiplication factor of E|e part
     
     bool valid = false;    // true if the number contains at least one digit
-    skipBlank(p);
-    const char* saved = p; // to restore position if we fail
     
+    skipBlank(p);
+    const char* saved = p;  // to restore position if we fail
+    
+    // Sign
     if (parseChar(p, '+')) {
         sign = 1.0;
     } else if (parseChar(p, '-')) {
         sign = -1.0;
     }
+    
+    // Integral part
     while (isdigit(*p)) {
         valid = true;
         ipart = ipart*10 + (*p - '0');
         p++;
     }
+    
+    // Possible decimal part
     if (parseChar(p, '.')) {
         while (isdigit(*p)) {
             valid = true;
@@ -177,8 +172,22 @@ static bool parseDouble(const char*& p, double& x)
             p++;
         }
     }
+    
+    // Possible E|e part
+    if (parseChar(p, 'E') || parseChar(p, 'e')) {
+        if (parseChar(p, '+')) {
+            expsign = 1.0;
+        } else if (parseChar(p, '-')) {
+            expsign = -1.0;
+        }
+        while (isdigit(*p)) {
+            expcoef = expcoef*10 + (*p - '0');
+            p++;
+        }
+    }
+    
     if (valid)  {
-        x = sign*(ipart + dpart/dcoef);
+        x = (sign*(ipart + dpart/dcoef)) * std::pow(10.0, expcoef*expsign);
     } else {
         p = saved;
     }
@@ -260,6 +269,7 @@ static bool parseMenuItem(const char*& p, std::string& name, double& value)
 static bool parseMenuItem2(const char*& p, std::string& name)
 {
     const char* saved = p;  // to restore position if we fail
+    // single quoted
     if (parseSQString(p, name)) {
         return true;
     } else {
@@ -326,7 +336,7 @@ static bool parseMenuList2(const char*& p, std::vector<std::string>& names, bool
     }
     
 error:
-    if (debug) { std::cerr << "parseMenuList2 : (" << saved << ") is not a valid list !\n"; }
+    if (debug) { fprintf(stderr, "parseMenuList2 : (%s) is not a valid list !\n", p); }
     p = saved;
     return false;
 }
@@ -355,6 +365,7 @@ static bool parseList(const char*& p, std::vector<std::string>& items)
 
 static bool parseMetaData(const char*& p, std::map<std::string, std::string>& metadatas)
 {
+    const char* saved = p; // to restore position if we fail
     std::string metaKey, metaValue;
     if (parseChar(p, ':') && parseChar(p, '[')) {
         do { 
@@ -364,12 +375,14 @@ static bool parseMetaData(const char*& p, std::map<std::string, std::string>& me
         } while (tryChar(p, ','));
         return parseChar(p, ']');
     } else {
+        p = saved;
         return false;
     }
 }
 
 static bool parseItemMetaData(const char*& p, std::vector<std::pair<std::string, std::string> >& metadatas)
 {
+    const char* saved = p; // to restore position if we fail
     std::string metaKey, metaValue;
     if (parseChar(p, ':') && parseChar(p, '[')) {
         do { 
@@ -379,6 +392,7 @@ static bool parseItemMetaData(const char*& p, std::vector<std::pair<std::string,
         } while (tryChar(p, ','));
         return parseChar(p, ']');
     } else {
+        p = saved;
         return false;
     }
 }
@@ -388,15 +402,17 @@ static bool parseItemMetaData(const char*& p, std::vector<std::pair<std::string,
 // "name" : "...", "inputs" : "...", "outputs" : "...", ...
 // and store the result as key/value
 /// ---------------------------------------------------------------------
-static bool parseGlobalMetaData(const char*& p, std::string& key, std::string& value, std::map<std::string, std::string>& metadatas, std::vector<std::string>& items)
+static bool parseGlobalMetaData(const char*& p, std::string& key, std::string& value, double& dbl, std::map<std::string, std::string>& metadatas, std::vector<std::string>& items)
 {
+    const char* saved = p; // to restore position if we fail
     if (parseDQString(p, key)) {
         if (key == "meta") {
             return parseMetaData(p, metadatas);
         } else {
-            return parseChar(p, ':') && (parseDQString(p, value) || parseList(p, items));
+            return parseChar(p, ':') && (parseDQString(p, value) || parseList(p, items) || parseDouble(p, dbl));
         }
     } else {
+        p = saved;
         return false;
     }
 }
@@ -406,12 +422,14 @@ static bool parseGlobalMetaData(const char*& p, std::string& key, std::string& v
 // "type" : "...", "label" : "...", "address" : "...", ...
 // and store the result in uiItems Vector
 /// ---------------------------------------------------------------------
-static bool parseUI(const char*& p, std::vector<itemInfo*>& uiItems, int& numItems)
+static bool parseUI(const char*& p, std::vector<itemInfo>& uiItems, int& numItems)
 {
+    const char* saved = p; // to restore position if we fail
     if (parseChar(p, '{')) {
    
         std::string label;
         std::string value;
+        double dbl = 0;
         
         do {
             if (parseDQString(p, label)) {
@@ -420,99 +438,92 @@ static bool parseUI(const char*& p, std::vector<itemInfo*>& uiItems, int& numIte
                         numItems++;
                     }
                     if (parseChar(p, ':') && parseDQString(p, value)) {   
-                        itemInfo* item = new itemInfo;
-                        item->type = value;
+                        itemInfo item;
+                        item.type = value;
                         uiItems.push_back(item);
                     }
                 }
                 
                 else if (label == "label") {
                     if (parseChar(p, ':') && parseDQString(p, value)) {
-                        itemInfo* item = uiItems[numItems];
-                        item->label = value;
+                        uiItems[numItems].label = value;
                     }
                 }
                 
                 else if (label == "url") {
                     if (parseChar(p, ':') && parseDQString(p, value)) {
-                        itemInfo* item = uiItems[numItems];
-                        item->url = value;
+                        uiItems[numItems].url = value;
                     }
                 }
                 
                 else if (label == "address") {
                     if (parseChar(p, ':') && parseDQString(p, value)) {
-                        itemInfo* item = uiItems[numItems];
-                        item->address = value;
+                        uiItems[numItems].address = value;
                     }
                 }
                 
                 else if (label == "index") {
-                    if (parseChar(p, ':') && parseDQString(p, value)) {
-                        itemInfo* item = uiItems[numItems];
-                        item->index = value;
+                    if (parseChar(p, ':') && parseDouble(p, dbl)) {
+                        uiItems[numItems].index = int(dbl);
                     }
                 }
                 
                 else if (label == "meta") {
-                    itemInfo* item = uiItems[numItems];
-                    if (!parseItemMetaData(p, item->meta)) {
+                    if (!parseItemMetaData(p, uiItems[numItems].meta)) {
                         return false;
                     }
                 }
                 
                 else if (label == "init") {
-                    if (parseChar(p, ':') && parseDQString(p, value)) {
-                        itemInfo* item = uiItems[numItems];
-                        item->init = value;
+                    if (parseChar(p, ':') && parseDouble(p, dbl)) {
+                        uiItems[numItems].init = dbl;
                     }
                 }
                 
                 else if (label == "min") {
-                    if (parseChar(p, ':') && parseDQString(p, value)) {
-                        itemInfo* item = uiItems[numItems];
-                        item->min = value;
+                    if (parseChar(p, ':') && parseDouble(p, dbl)) {
+                        uiItems[numItems].fmin = dbl;
                     }
                 }
                 
                 else if (label == "max") {
-                    if (parseChar(p, ':') && parseDQString(p, value)) {
-                        itemInfo* item = uiItems[numItems];
-                        item->max = value;
+                    if (parseChar(p, ':') && parseDouble(p, dbl)) {
+                        uiItems[numItems].fmax = dbl;
                     }
                 }
                 
-                else if (label == "step"){
-                    if (parseChar(p, ':') && parseDQString(p, value)) {
-                        itemInfo* item = uiItems[numItems];
-                        item->step = value;
+                else if (label == "step") {
+                    if (parseChar(p, ':') && parseDouble(p, dbl)) {
+                        uiItems[numItems].step = dbl;
                     }
                 }
                 
                 else if (label == "items") {
                     if (parseChar(p, ':') && parseChar(p, '[')) {
-                        do { 
+                        do {
                             if (!parseUI(p, uiItems, numItems)) {
+                                p = saved;
                                 return false;
                             }
                         } while (tryChar(p, ','));
                         if (parseChar(p, ']')) {
-                            itemInfo* item = new itemInfo;
-                            item->type = "close";
+                            itemInfo item;
+                            item.type = "close";
                             uiItems.push_back(item);
                             numItems++;
                         }
                     }
                 }
             } else {
+                p = saved;
                 return false;
             }
             
         } while (tryChar(p, ','));
-        
+    
         return parseChar(p, '}');
     } else {
-        return false;
+        return true; // "items": [] is valid
     }
 }
 
@@ -522,22 +533,29 @@ static bool parseUI(const char*& p, std::vector<itemInfo*>& uiItems, int& numIte
 //
 // and store the result in map Metadatas and vector containing the items of the interface. Returns true if parsing was successfull.
 /// ---------------------------------------------------------------------
-static bool parseJson(const char*& p, std::map<std::string, std::string>& metadatas1, std::map<std::string, std::vector<std::string> >& metadatas2, std::vector<itemInfo*>& uiItems)
+static bool parseJson(const char*& p,
+                      std::map<std::string, std::pair<std::string, double> >& metaDatas0,
+                      std::map<std::string, std::string>& metaDatas1,
+                      std::map<std::string, std::vector<std::string> >& metaDatas2,
+                      std::vector<itemInfo>& uiItems)
 {
     parseChar(p, '{');
     
     do {
         std::string key;
         std::string value;
+        double dbl = 0;
         std::vector<std::string> items;
-        if (parseGlobalMetaData(p, key, value, metadatas1, items)) {
+        if (parseGlobalMetaData(p, key, value, dbl, metaDatas1, items)) {
             if (key != "meta") {
                 // keep "name", "inputs", "outputs" key/value pairs
                 if (items.size() > 0) {
-                    metadatas2[key] = items;
+                    metaDatas2[key] = items;
                     items.clear();
+                } else if (value != "") {
+                    metaDatas0[key].first = value;
                 } else {
-                    metadatas1[key] = value;
+                    metaDatas0[key].second = dbl;
                 }
             }
         } else if (key == "ui") {
@@ -550,3 +568,4 @@ static bool parseJson(const char*& p, std::map<std::string, std::string>& metada
 }
 
 #endif // SIMPLEPARSER_H
+/**************************  END  SimpleParser.h **************************/
