@@ -42,6 +42,7 @@
 
 #include <cstddef>
 #include <string>
+#include <iostream>
 #include <math.h>
 #include <strings.h>
 #include <cstdlib>
@@ -374,7 +375,7 @@ BelaOSCUI gOSCUI(OSC_IP_ADDRESS, OSC_IN_PORT, OSC_OUT_PORT);
 #endif
 
 #ifdef HTTPDGUI
-httpdUI* gHttpdInterface;
+httpdUI* gHttpdInterface = NULL;
 #endif
 
 #ifdef SOUNDFILE
@@ -419,7 +420,7 @@ bool setup(BelaContext* context, void* userData)
     // Polyphonic with effect
 #ifdef POLY2
     int group = 1;
-    std::cout << "Started with " << nvoices << " voices\n";
+    cout << "Started with " << nvoices << " voices" << endl;
     gDSP = new mydsp_poly(new mydsp(), nvoices, true, group);
     
 #ifdef MIDICTRL
@@ -436,12 +437,12 @@ bool setup(BelaContext* context, void* userData)
     int group = 1;
     
     if (nvoices > 0) {
-        std::cout << "Started with " << nvoices << " voices\n";
+        cout << "Started with " << nvoices << " voices" << endl;
         gDSP = new mydsp_poly(new mydsp(), nvoices, true, group);
         
 #ifdef MIDICTRL
         if (midi_sync) {
-            gDSP = new timed_dsp(DSP);
+            gDSP = new timed_dsp(gDSP);
         }
 #endif
     } else {
@@ -458,7 +459,7 @@ bool setup(BelaContext* context, void* userData)
 #endif
     
     if (gDSP == 0) {
-        std::cerr << "Unable to allocate Faust DSP object" << std::endl;
+        cerr << "Unable to allocate Faust DSP object" << endl;
         return false;
     }
     
